from .dev_file import DEV_FILE_CONTENT
from .main_file import MAIN_FILE_CONTENT
from .common_api_file import COMMON_API_CONTENT, COMMON_API_INIT
from .core_file import CORE_FILE_CONTENT
from .template_file import TEMPLATE_FILE_CONTENT
from .env_file import ENV_FILE_CONTENT

__all__ = [
    DEV_FILE_CONTENT,
    MAIN_FILE_CONTENT,
    COMMON_API_CONTENT,
    COMMON_API_INIT,
    CORE_FILE_CONTENT,
    TEMPLATE_FILE_CONTENT,
    ENV_FILE_CONTENT,
]



from builders_hut.cmd_interface import interface


def main():
    try:
        interface()
    except Exception as e:
        print(f"Project setup failed: {e}")


if __name__ == "__main__":
    main()



from pathlib import Path
from builders_hut.setups.base_setup import BaseSetup
from builders_hut.setups.file_contents import (
    DEV_FILE_CONTENT,
    MAIN_FILE_CONTENT,
    COMMON_API_CONTENT,
    COMMON_API_INIT,
    CORE_FILE_CONTENT,
    TEMPLATE_FILE_CONTENT,
    ENV_FILE_CONTENT,
)


class SetupFileWriter(BaseSetup):
    FILES_TO_WRITE: dict[Path, str] = {
        Path("app/main.py"): MAIN_FILE_CONTENT,
        Path("app/scripts/dev.py"): DEV_FILE_CONTENT,
        Path("app/api/common.py"): COMMON_API_CONTENT,
        Path("app/api/__init__.py"): COMMON_API_INIT,
        Path("app/core/config.py"): CORE_FILE_CONTENT,
        Path("app/templates/index.html"): TEMPLATE_FILE_CONTENT,
        Path(".env"): ENV_FILE_CONTENT,
    }

    def create(self):
        self._write_files()

    def _write_files(self) -> None:
        for path, content in self.FILES_TO_WRITE.items():
            if path.name == ".env":
                content = content.format(
                    title=self.name,
                    description=self.description,
                    version=self.version,
                )
            path = self.location / path
            self._write_file(path, content)

    @staticmethod
    def _write_file(path: Path, content: str) -> None:
        if not path.exists():
            raise FileNotFoundError(f"{path} does not exist")

        path.write_text(content, encoding="utf-8")

    def configure(self, name: str, description: str, version: str):
        self.name = name
        self.description = description
        self.version = version



import subprocess
from rich import print

from builders_hut.setups.base_setup import BaseSetup
from builders_hut.utils import get_platform, write_pyproject

PACKAGES = [
    "fastapi",
    "python-dotenv",
    "email-validator",
    "tzdata",
    "pydantic-settings",
    "scalar-fastapi",
    "uvicorn",
    "jinja2"
]

DEV_PACKAGES = ["pytest"]


class SetupEnv(BaseSetup):
    """Create Env and Install Base Packages"""

    def create(self):
        try:
            platform = get_platform()

            print(
                f"Working On: [bold green]{platform[0].upper()}{platform[1:]}[/bold green]"
            )

            print(f"Working Directory: {self.location}")

            print("Updating pyproject file...")

            write_pyproject(
                path=(self.location / "pyproject.toml"),
                name=self.name,
                description=self.description,
                version=self.version,
                dependencies=PACKAGES,
                dev_dependencies=DEV_PACKAGES,
            )

            print("Creating Environment...")

            subprocess.run(
                "python -m venv .venv",
                cwd=self.location,
                shell=True,
                check=True,
                stderr=subprocess.DEVNULL,
                stdout=subprocess.DEVNULL,
            )

            print("Installing Packages...")

            command = "pip install -e ."
            python_file = (
                ".venv/bin/python -m"
                if platform == "linux"
                else ".venv\\Scripts\\python.exe -m"
            )

            full_command = f"{python_file} {command}"

            subprocess.run(
                full_command,
                cwd=self.location,
                shell=True,
                check=True,
                stderr=subprocess.DEVNULL,
                stdout=subprocess.DEVNULL,
            )

            print("[bold green]Done [/bold green]")

        except Exception as e:
            print(str(e))
            raise RuntimeError("Failed to create environment") from e

    def configure(self, name: str, description: str, version: str):
        self.name = name
        self.description = description
        self.version = version



from .env_setup import SetupEnv
from .files_setup import SetupFiles
from .structure_setup import SetupStructure
from .file_writter import SetupFileWriter


__all__ = [SetupEnv, SetupFiles, SetupStructure, SetupFileWriter]



from textwrap import dedent

CORE_FILE_CONTENT = dedent("""
from pydantic_settings import BaseSettings, SettingsConfigDict
from dotenv import load_dotenv
from pydantic import Field
from functools import lru_cache

load_dotenv()


class Settings(BaseSettings):
    '''
    Settings for the application
    '''
    model_config = SettingsConfigDict(
        env_file=".env", env_file_encoding="utf-8", extra="ignore"
    )

    TITLE: str = Field(
        description="The title of the application", validation_alias="TITLE"
    )
    DESCRIPTION: str = Field(
        description="The description of the application",
        validation_alias="DESCRIPTION",
    )
    VERSION: str = Field(
        description="The version of the application", validation_alias="VERSION"
    )
    DEBUG: bool = Field(
        description="Whether the application is in debug mode",
        validation_alias="DEBUG",
    )
    PORT: int = Field(
        description="The port to run the application on", validation_alias="PORT"
    )
    HOST: str = Field(
        description="The host to run the application on", validation_alias="HOST"
    )


@lru_cache(maxsize=1)
def get_settings() -> Settings:
    '''
    Get the settings
    '''
    return Settings()


settings = get_settings()

""")



import shutil
from pathlib import Path

from builders_hut.setups import SetupEnv, SetupFiles, SetupFileWriter, SetupStructure
from builders_hut.utils import setup_project


def interface():
    try:
        current_path = Path(__name__).resolve().parent

        current_path = Path(current_path) / "demo"

        project_location = current_path

        if project_location.exists():
            """ clear existing """
            shutil.rmtree(project_location)

        setup_to_do = [SetupStructure, SetupFiles, SetupEnv, SetupFileWriter]

        name = input("Enter project title: ").strip()
        if not name:
            name = current_path.name.split("/")[-1]

        description = input("Enter project description: ").strip()
        if not description:
            description = "A new project"

        version = input("Enter project version (default: 0.1.0): ").strip()
        if not version:
            version = "0.1.0"

        for setup in setup_to_do:
            setup_project(
                project_location,
                setup,
                name=name,
                description=description,
                version=version,
            )

        print("Project setup completed successfully.")
    except Exception as e:
        print(f"Project setup failed: {e}")
